@@ -14,12 +14,15 @@
 # The Original Code is Mozprofile.
 #
 # The Initial Developer of the Original Code is
-# Mozilla Corporation.
+#   The Mozilla Foundation.
 # Portions created by the Initial Developer are Copyright (C) 2011
 # the Initial Developer. All Rights Reserved.
 #
 # Contributor(s):
-#  Andrew Halberstadt <halbersa@gmail.com>
+#   Andrew Halberstadt <halbersa@gmail.com>
+#   Mikeal Rogers <mikeal.rogers@gmail.com>
+#   Clint Talbert <ctalbert@mozilla.com>
+#   Jeff Hammel <jhammel@mozilla.com>
 #
 # Alternatively, the contents of this file may be used under the terms of
 # either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,14 +38,14 @@
 #
 # ***** END LICENSE BLOCK *****
 
-import urllib2
 import os
-import sys
+import shutil
 import tempfile
+import urllib2
 import zipfile
-from xml.dom import minidom
-from manifestparser import ManifestParser
 from distutils import dir_util
+from manifestparser import ManifestParser
+from xml.dom import minidom
 
 # Needed for the AMO's rest API - https://developer.mozilla.org/en/addons.mozilla.org_%28AMO%29_API_Developers%27_Guide/The_generic_AMO_API
 AMO_API_VERSION = "1.5"
@@ -58,10 +61,6 @@ class AddonManager(object):
         """
         self.profile = profile
         self.installed_addons = []
-        # keeps track of addons and manifests that were passed to install_addons
-        self.addons = []
-        self.manifests = []
-        
 
     def install_addons(self, addons=None, manifests=None):
         """
@@ -81,14 +80,13 @@ class AddonManager(object):
                 manifests = [manifests]
             for manifest in manifests:
                 self.install_from_manifest(manifest)
-        
+
 
     def install_from_manifest(self, filepath):
         """
         Installs addons from a manifest
         filepath - path to the manifest of addons to install
         """
-        self.manifests.append(filepath)
         manifest = ManifestParser()
         manifest.read(filepath)
         addons = manifest.get()
@@ -100,7 +98,7 @@ class AddonManager(object):
 
             # No path specified, try to grab it off AMO
             locale = addon.get('amo_locale', 'en_US')
-    
+
             query = 'https://services.addons.mozilla.org/' + locale + '/firefox/api/' + AMO_API_VERSION + '/'
             if 'amo_id' in addon:
                 query += 'addon/' + addon['amo_id']                 # this query grabs information on the addon base on its id
@@ -112,7 +110,7 @@ class AddonManager(object):
     @classmethod
     def get_amo_install_path(self, query):
         """
-        Return the addon xpi install path for the specified AMO query. 
+        Return the addon xpi install path for the specified AMO query.
         See: https://developer.mozilla.org/en/addons.mozilla.org_%28AMO%29_API_Developers%27_Guide/The_generic_AMO_API
         for query documentation.
         """
@@ -121,50 +119,72 @@ class AddonManager(object):
         for node in dom.getElementsByTagName('install')[0].childNodes:
             if node.nodeType == node.TEXT_NODE:
                 return node.data
- 
+
     @classmethod
-    def get_addon_id(self, addon_path):
+    def addon_details(cls, addon_path):
         """
-        return the id for a given addon, or None if not found
+        returns a dictionary of details about the addon
         - addon_path : path to the addon directory
+        Returns:
+        {'id':      u'rainbow@colors.org', # id of the addon
+         'version': u'1.4',                # version of the addon
+         'name':    u'Rainbow',            # name of the addon
+         'unpack':  False } # whether to unpack the addon
         """
-        
-        def find_id(desc):
-            """finds the addon id give its description"""
-            
-            addon_id = None
-            for elem in desc:
-                apps = elem.getElementsByTagName('em:targetApplication')
-                apps.extend(elem.getElementsByTagName('targetApplication'))
-                if apps:
-                    for app in apps:
-                        # remove targetApplication nodes, they contain id's we aren't interested in
-                        elem.removeChild(app)
 
-                    # find the id tag
-                    if elem.getElementsByTagName('em:id'):
-                        addon_id = str(elem.getElementsByTagName('em:id')[0].firstChild.data)
-                    elif elem.hasAttribute('em:id'):
-                        addon_id = str(elem.getAttribute('em:id'))
-                    elif elem.getElementsByTagName('id'):
-                        addon_id = str(elem.getElementsByTagName('id')[0].firstChild.data)
-                    
-            return addon_id
+        # TODO: We don't use the unpack variable yet, but we should: bug 662683
+        details = {
+            'id': None,
+            'unpack': False,
+            'name': None,
+            'version': None
+        }
 
-        doc = minidom.parse(os.path.join(addon_path, 'install.rdf')) 
+        def get_namespace_id(doc, url):
+            attributes = doc.documentElement.attributes
+            namespace = ""
+            for i in range(attributes.length):
+                if attributes.item(i).value == url:
+                    if ":" in attributes.item(i).name:
+                        # If the namespace is not the default one remove 'xlmns:'
+                        namespace = attributes.item(i).name.split(':')[1] + ":"
+                        break
+            return namespace
 
-        for tag in 'Description', 'RDF:Description':
-            desc = doc.getElementsByTagName(tag)
-            addon_id = find_id(desc)
-            if addon_id:
-                return addon_id
+        def get_text(element):
+            """Retrieve the text value of a given node"""
+            rc = []
+            for node in element.childNodes:
+                if node.nodeType == node.TEXT_NODE:
+                    rc.append(node.data)
+            return ''.join(rc).strip()
 
-    def install_from_path(self, path):
+        doc = minidom.parse(os.path.join(addon_path, 'install.rdf'))
+
+        # Get the namespaces abbreviations
+        em = get_namespace_id(doc, "http://www.mozilla.org/2004/em-rdf#")
+        rdf = get_namespace_id(doc, "http://www.w3.org/1999/02/22-rdf-syntax-ns#")
+
+        description = doc.getElementsByTagName(rdf + "Description").item(0)
+        for node in description.childNodes:
+            # Remove the namespace prefix from the tag for comparison
+            entry = node.nodeName.replace(em, "")
+            if entry in details.keys():
+                details.update({ entry: get_text(node) })
+
+        # turn unpack into a true/false value
+        if isinstance(details['unpack'], basestring):
+            details['unpack'] = details['unpack'].lower() == 'true'
+
+        return details
+
+    def install_from_path(self, path, unpack=False):
         """
-        Installs addon from a filepath, url 
+        Installs addon from a filepath, url
         or directory of addons in the profile.
+        - path: url, path to .xpi, or directory of addons
+        - unpack: whether to unpack unless specified otherwise in the install.rdf
         """
-        self.addons.append(path)
 
         # if the addon is a url, download it
         # note that this won't work with protocols urllib2 doesn't support
@@ -180,12 +200,13 @@ class AddonManager(object):
         # if the addon is a directory, install all addons in it
         addons = [path]
         if not path.endswith('.xpi') and not os.path.exists(os.path.join(path, 'install.rdf')):
-            assert os.path.isdir(path)
+            assert os.path.isdir(path), "Addon '%s' cannot be installed" % path
             addons = [os.path.join(path, x) for x in os.listdir(path)]
 
         # install each addon
         for addon in addons:
             tmpdir = None
+            xpifile = None
             if addon.endswith('.xpi'):
                 tmpdir = tempfile.mkdtemp(suffix = '.' + os.path.split(addon)[-1])
                 compressed_file = zipfile.ZipFile(addon, 'r')
@@ -199,16 +220,24 @@ class AddonManager(object):
                         f = open(os.path.join(tmpdir, name), 'wb')
                         f.write(data)
                         f.close()
+                xpifile = addon
                 addon = tmpdir
 
             # determine the addon id
-            addon_id = AddonManager.get_addon_id(addon)
-            assert addon_id is not None, 'The addon id could not be found: %s' % addon
- 
+            addon_details = AddonManager.addon_details(addon)
+            addon_id = addon_details.get('id')
+            assert addon_id, 'The addon id could not be found: %s' % addon
+
             # copy the addon to the profile
-            addon_path = os.path.join(self.profile, 'extensions', addon_id)
-            dir_util.copy_tree(addon, addon_path, preserve_symlinks=1)
-            self.installed_addons.append(addon_path)
+            extensions_path = os.path.join(self.profile, 'extensions')
+            addon_path = os.path.join(extensions_path, addon_id)
+            if not unpack and not addon_details['unpack'] and xpifile:
+                if not os.path.exists(extensions_path):
+                    os.makedirs(extensions_path)
+                shutil.copy(xpifile, addon_path + '.xpi')
+            else:
+                dir_util.copy_tree(addon, addon_path, preserve_symlinks=1)
+                self.installed_addons.append(addon_path)
 
             # remove the temporary directory, if any
             if tmpdir:
@@ -217,7 +246,7 @@ class AddonManager(object):
         # remove temporary file, if any
         if tmpfile:
             os.remove(tmpfile)
-    
+
     def clean_addons(self):
         """Cleans up addons in the profile."""
         for addon in self.installed_addons:
